/*
  simple - a formal system demonstrating cyclic values
  Copyright (C) 1996  Scott Draves <spot@cs.cmu.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundationeither version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this programif not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "time.h"
#include "runtime.h"
#include "names.h"



int sum_reduce_hand(int *start, int *stop, int sum) {
   int v;

   while(start != stop) {
     v = *start;
     sum += (((v>>0)&255) +
	     ((v>>8)&255) +
	     ((v>>16)&255) +
	     ((v>>24)&255));
     start += 1;
   }
   return sum;
}

void iota(int *start, int *stop) {
  int i = 0;
  while(start != stop) {	
    *start++ = i | ((i+1)<<8) | ((i+2)<<16) | ((i+3)<<24);
    i+=4;
  }
}

void int_copy(int *start0, int* stop0, int *start1, int* stop1) {
  while (start0 != stop0)
    *start0++ = *start1++;
}

void gap_copy(int *start0, int* stop0, int *start1, int* stop1) {
  while (start0 != stop0) {
    int v = *start0;
    int b0 = (v>>0)&255;
    int b1 = (v>>8)&255;
    int b2 = (v>>16)&255;
    int b3 = (v>>24)&255;
    int mask = 0xff00ff00;
    start1[0] = (start1[0] & mask) | b0 | (b1 << 16);
    start1[1] = (start1[1] & mask) | b2 | (b3 << 16);
    start0++;
    start1+=2;
  }
}

int
nyb_loop_static12(int *start, int *stop) {
   int sum = 0;

   while (start != stop) {
      int w0 = start[0];
      int w1 = start[1];
      int w2 = start[2];
      sum += ((w0 & 0xfff) +
	      ((w0 >> 12) & 0xfff) +
	      (((w0 >> 24) & 0xff) | ((w1 & 0xf) << 8)) +
	      ((w1 >> 4) & 0xfff) +
	      ((w1 >> 16) & 0xfff) +
	      (((w1 >> 28) & 0xf) | ((w2 & 0xff) << 4)) +
	      ((w2 >> 8) & 0xfff) +
	      ((w2 >> 20) & 0xfff));
      start += 3;
   }
   return sum;
}

#define enc(x) x
int
base64_encode(int *start0, int *stop0, int *start1, int *stop1) {
  while (start0 != stop0) {
      int w0 = start0[0];
      int w1 = start0[1];
      int w2 = start0[2];
      start1[0] = ((enc(w0 & 0x3f)) |
		   ((enc((w0>>6) & 0x3f)) << 8) |
		   ((enc((w0>>12) & 0x3f)) << 16) |
		   ((enc((w0>>18) & 0x3f)) << 24));
      start1[1] = ((enc((w0>>24) & 0x3f)) |
		   ((enc(((w0>>30)&0x3f)|((w1<<2)&0x3c))) << 8) |
		   ((enc((w1>>4) & 0x3f)) << 16) |
		   ((enc((w1>>10) & 0x3f)) << 24));
      start1[2] = ((enc((w1>>16) & 0x3f)) |
		   ((enc((w1>>22) & 0x3f)) << 8) |
		   ((enc(((w1>>28)&0x3f)|((w2<<4)&0x30))) << 16) |
		   ((enc((w2>>2) & 0x3f)) << 24));
      start1[3] = ((enc((w2>>8) & 0x3f)) |
		   ((enc((w2>>14) & 0x3f)) << 8) |
		   ((enc((w2>>20) & 0x3f)) << 16) |
		   ((enc((w2>>26) & 0x3f)) << 24));
      start0 += 3;
      start1 += 4;
  }
}
#define dec(x) x
int base64_decode(int *start0, int *stop0, int *start1, int *stop1) {
  while (start0 != stop0) {
      int w0 = start0[0];
      int w1 = start0[1];
      int w2 = start0[2];
      int w3 = start0[3];

      start1[0] = (dec(w0&0xff) |
		   (dec((w0>>8)&0xff) << 6) |
		   (dec((w0>>16)&0xff) << 12) |
		   (dec((w0>>24)&0xff) << 18) |
		   (dec((w1)&0xff) << 24) |
		   (dec((w1)&0xff) << 30));
      start1[1] = ((dec((w1)&0xff) >> 2) |
		   (dec((w1>>8)&0xff) << 4) |
		   (dec((w1>>16)&0xff) << 10) |
		   (dec((w1>>24)&0xff) << 16) |
		   (dec((w2)&0xff) << 22) |
		   (dec((w2>>8)&0xff) << 28));
      start1[2] = ((dec((w2>>16)&0xff) >> 4) |
		   (dec((w2>>24)&0xff) << 2) |
		   (dec((w3)&0xff) << 8) |
		   (dec((w3>>8)&0xff) << 14) |
		   (dec((w3>>16)&0xff) << 20) |
		   (dec((w3>>24)&0xff) << 26));

      start0 += 4;
      start1 += 3;
  }  
}


main(int argc, char **argv)
{
  int n;
  int s2;
  int v[1000];
  int w[2000];
  int iters = 1000;
  int v_len = 1000;
  int ddddum;
  int nsamps;

  word_t start = ((word_t) v>>2);
  word_t start1 = ((word_t) w>>2);
  word_t dummy = ((int)&ddddum)>>2;

  s2 = 0;
  begin_timer();
  for (n = 0; n < iters; n++) {
    s2 += s_r_s(start, start+v_len, 0, 0, 0);
  }
  printf("a sum %d %.3g\n", s2, end_timer());

  s2 = 0;
  begin_timer();
  for (n = 0; n < iters; n++) {
    s2 += sum_reduce_hand(v, v+v_len, 0);
  }
  printf("m sum %d %.3g\n", s2, end_timer());


  begin_timer();
  for (n = 0; n < iters; n++) {
    i_s(start, start+v_len, 0, dummy, 0);
  }
  printf("a iota %.3g\n", end_timer());

  begin_timer();
  for (n = 0; n < iters; n++) {
    iota(v, v+v_len);
  }
  printf("m iota %.3g\n", end_timer());

  begin_timer();
  for (n = 0; n < iters; n++) {
    c_s(start, start+v_len, start1, start1+v_len, 0, dummy, 0, dummy, 0);
  }
  printf("a copy %.3g\n", end_timer());

  begin_timer();
  for (n = 0; n < iters; n++) {
    int_copy(v, v+v_len, w, w+v_len);
  }
  printf("m copy %.3g\n", end_timer());

  begin_timer();
  for (n = 0; n < iters; n++) {
    c_s_g(start, start+v_len, start1, start1+2*v_len, 0, dummy, 0, dummy, 0);
  }
  printf("a copy_gaps %.3g\n", end_timer());

  begin_timer();
  for (n = 0; n < iters; n++) {
    gap_copy(v, v+v_len, w, w+2*v_len);
  }
  printf("m copy_gaps %.3g\n", end_timer());

  s2 = 0;
  begin_timer();
  for (n = 0; n < iters; n++) {
    s2 += s_r_12(start, start+make_even(v_len,3), 0, 0, 0, 0, 0);
  }
  printf("a sum 12 %d %.3g\n", s2, end_timer());

  s2 = 0;
  begin_timer();
  for (n = 0; n < iters; n++) {
    s2 += nyb_loop_static12(v, v+make_even(v_len,3));
  }
  printf("m sum 12 %d %.3g\n", s2, end_timer());

  nsamps = (make_even(v_len, 12) * 32) / 8;
  begin_timer();
  for (n = 0; n < iters; n++) {
    c_s_6_8(start, start+(nsamps*6)/32, start1, start1+(nsamps*8)/32, 0, dummy, 0, dummy, 0, dummy, 0);
  }
  printf("a cs68 %.3g\n", end_timer());

  nsamps = (make_even(v_len, 12) * 32) / 8;
  begin_timer();
  for (n = 0; n < iters; n++) {
    base64_encode(v, v+(nsamps*6)/32, w, w+(nsamps*8)/32);
  }
  printf("m cs68 %.3g\n", end_timer());

  nsamps = (make_even(v_len, 12) * 32) / 8;
  begin_timer();
  for (n = 0; n < iters; n++) {
    c_s_8_6(start, start+(nsamps*8)/32, start1, start1+(nsamps*6)/32, 0, dummy, 0, dummy, 0, dummy, 0);
  }
  printf("a cs86 %.3g\n", end_timer());

  nsamps = (make_even(v_len, 12) * 32) / 8;
  begin_timer();
  for (n = 0; n < iters; n++) {
    base64_decode(v, v+(nsamps*8)/32, w, w+(nsamps*6)/32);
  }
  printf("m cs86 %.3g\n", end_timer());
}
