/*
  simple - a formal system demonstrating cyclic values
  Copyright (C) 1996  Scott Draves <spot@cs.cmu.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundationeither version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this programif not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "time.h"
#include "runtime.h"
#include "names.h"

void rgb2m_1_m(int *s0, int *e0, int *s1, int *e1)
{
  while (s0 != e0) {
    int w0 = s0[0];
    int r0 = w0 & 0xff;
    int g0 = (w0>>8) & 0xff;
    int b0 = (w0>>16) & 0xff;
    int o0 = (30*r0 + 25*g0 + 9*b0)>>6;
    int w1 = s0[1];
    int r1 = w1 & 0xff;
    int g1 = (w1>>8) & 0xff;
    int b1 = (w1>>16) & 0xff;
    int o1 = (30*r1 + 25*g1 + 9*b1)>>6;
    int w2 = s0[2];
    int r2 = w2 & 0xff;
    int g2 = (w2>>8) & 0xff;
    int b2 = (w2>>16) & 0xff;
    int o2 = (30*r2 + 25*g2 + 9*b2)>>6;
    int w3 = s0[3];
    int r3 = w3 & 0xff;
    int g3 = (w3>>8) & 0xff;
    int b3 = (w3>>16) & 0xff;
    int o3 = (30*r3 + 25*g3 + 9*b3)>>6;
    *s1 = o0 | (o1 << 8) | (o2 << 16) | (o3 << 24);
    s0 += 4;
    s1 += 1;
  }
}

void rgb2m_2_m(int *s0, int *e0, int *s1, int *e1)
{
  while (s0 != e0) {
    int w0 = s0[0];
    int r0 = w0 & 0xff;
    int g0 = (w0>>8) & 0xff;
    int b0 = (w0>>16) & 0xff;
    int r1 = (w0>>24) & 0xff;
    int w1 = s0[1];
    int g1 = w1 & 0xff;
    int b1 = (w1>>8) & 0xff;
    int r2 = (w1>>16) & 0xff;
    int g2 = (w1>>24) & 0xff;
    int w2 = s0[2];
    int b2 = w2 & 0xff;
    int r3 = (w2>>8) & 0xff;
    int g3 = (w2>>16) & 0xff;
    int b3 = (w2>>24) & 0xff;

    int o0 = (30*r0 + 25*g0 + 9*b0)>>6;
    int o1 = (30*r1 + 25*g1 + 9*b1)>>6;
    int o2 = (30*r2 + 25*g2 + 9*b2)>>6;
    int o3 = (30*r3 + 25*g3 + 9*b3)>>6;
    *s1 = o0 | (o1 << 8) | (o2 << 16) | (o3 << 24);
    s0 += 3;
    s1 += 1;
  }
}

void rgb2m_3_m(int *s0, int *e0, int *s1, int *e1,
	       int *s2, int *e2, int *s3, int *e3)
{
  while (s0 != e0) {
    int rw = s0[0];
    int r0 = rw & 0xff;
    int r1 = (rw>>8) & 0xff;
    int r2 = (rw>>16) & 0xff;
    int r3 = (rw>>16) & 0xff;

    int gw = s1[0];
    int g0 = gw & 0xff;
    int g1 = (gw>>8) & 0xff;
    int g2 = (gw>>16) & 0xff;
    int g3 = (gw>>16) & 0xff;

    int bw = s2[0];
    int b0 = bw & 0xff;
    int b1 = (bw>>8) & 0xff;
    int b2 = (bw>>16) & 0xff;
    int b3 = (bw>>16) & 0xff;

    int o0 = (30*r0 + 25*g0 + 9*b0)>>6;
    int o1 = (30*r1 + 25*g1 + 9*b1)>>6;
    int o2 = (30*r2 + 25*g2 + 9*b2)>>6;
    int o3 = (30*r3 + 25*g3 + 9*b3)>>6;
    *s3 = o0 | (o1 << 8) | (o2 << 16) | (o3 << 24);

    s0 += 1;
    s1 += 1;
    s2 += 1;
    s3 += 1;
  }
}


void rgb2m_3_b(int *s0, int *e0, int *s1, int *e1,
	       int *s2, int *e2, int *s3, int *e3)
{
  char *r, *g, *b, *m, *re;
  r = (char *) s0;
  re = (char *) e0;
  g = (char *) s1;
  b = (char *) s2;
  m = (char *) s3;
  while (r != re) {
    int r0 = r[0];
    int r1 = r[1];
    int r2 = r[2];
    int r3 = r[3];

    int g0 = g[0];
    int g1 = g[1];
    int g2 = g[2];
    int g3 = g[3];

    int b0 = b[0];
    int b1 = b[1];
    int b2 = b[2];
    int b3 = b[3];

    int o0 = (30*r0 + 25*g0 + 9*b0)>>6;
    int o1 = (30*r1 + 25*g1 + 9*b1)>>6;
    int o2 = (30*r2 + 25*g2 + 9*b2)>>6;
    int o3 = (30*r3 + 25*g3 + 9*b3)>>6;

    m[0] = o0;
    m[1] = o1;
    m[2] = o2;
    m[3] = o3;

    r += 4;
    g += 4;
    b += 4;
    m += 4;
  }
}

#define disable_gen 1

main(int argc, char **argv)
{
  int n;
  int s2;
  int v[1000];
  int w[2000];
  int iters = 1000;
  int v_len = 1000;
  int ddddum;
  int nsamps;
  int wid;

  word_t start = ((word_t) v>>2);
  word_t start1 = ((word_t) w>>2);
  word_t dummy = ((int)&ddddum)>>2;
#if !disable_gen
  rgb2m_1(start, start+v_len, start1, start1+v_len/4,
	  0, dummy, 0, dummy, 0);

  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_1(start, start+v_len, start1, start1+v_len/4,
	    0, dummy, 0, dummy, 0);
  }
  printf("a rgb2m_1 %.3g\n", end_timer());
#endif
  rgb2m_1_m(v, v+v_len, w, w+v_len/4);

  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_1_m(v, v+v_len, w, w+v_len/4);
  }
  printf("m rgb2m_1 %.3g\n", end_timer());

  nsamps = (make_even(v_len, 3) * 32) / 24;
#if !disable_gen
  rgb2m_2(start, start+(nsamps*24)/32, start1, start1+(nsamps*8)/32,
	  0, dummy, 0, dummy, 0, dummy, 0);

  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_2(start, start+(nsamps*24)/32, start1, start1+(nsamps*8)/32,
	    0, dummy, 0, dummy, 0, dummy, 0);
  }
  printf("a rgb2m_2 %.3g\n", end_timer());
#endif

  rgb2m_2_m(v, v+(nsamps*24)/32, w, w+(nsamps*8)/32);

  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_2_m(v, v+(nsamps*24)/32, w, w+(nsamps*8)/32);
  }
  printf("m rgb2m_2 %.3g\n", end_timer());

  nsamps = (v_len*4)/3;
  wid = nsamps/4;

#if !disable_gen
  rgb2m_3(start, start+wid,
	  start+wid, start+2*wid,
	  start+2*wid, start+3*wid,
	  start1, start1+wid,
	  0, dummy, 0, dummy, 0, dummy, 0, dummy, 0);
  
  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_3(start, start+wid,
	    start+wid, start+2*wid,
	    start+2*wid, start+3*wid,
	    start1, start1+wid,
	    0, dummy, 0, dummy, 0, dummy, 0, dummy, 0);
  }
  printf("a rgb2m_3 %.3g\n", end_timer());
#endif

  rgb2m_3_m(v, v+wid, v+wid, v+2*wid,
	    v+2*wid, v+3*wid, w, w+wid);
	    
  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_3_m(v, v+wid, v+wid, v+2*wid,
	      v+2*wid, v+3*wid, w, w+wid);
  }
  printf("m rgb2m_3 %.3g\n", end_timer());

  rgb2m_3_b(v, v+wid, v+wid, v+2*wid,
	    v+2*wid, v+3*wid, w, w+wid);
	    
  begin_timer();
  for (n = 0; n < iters; n++) {
    rgb2m_3_b(v, v+wid, v+wid, v+2*wid,
	      v+2*wid, v+3*wid, w, w+wid);
  }
  printf("b rgb2m_3 %.3g\n", end_timer());
}
