(*
  simple - a formal system demonstrating cyclic values
  Copyright (C) 1996  Scott Draves <spot@cs.cmu.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundationeither version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this programif not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)

functor Parse(structure Token  : TOKEN
	      structure Syntax : SYNTAX
	      sharing type Token.Name = Syntax.Name) : PARSE =
struct

structure Token  = Token
structure Syntax = Syntax
open Token Syntax

(* like mini eval *)
fun make_pattern (exp_prim(p_pair,[l,r])) = m_pair(make_pattern(l), make_pattern(r))
  | make_pattern (exp_var(v)) = m_dynamic(exp_var(v))
  | make_pattern (exp_atom(a)) = m_static(a)
  | make_pattern (exp_prim(p_plus,[exp_prim(p_times,[exp_atom(atom_int(b)),q]),exp_atom(atom_int(r))])) = m_cyclic(b,q,r)
  | make_pattern e = (print ("bad pattern: " ^ (unparse e) ^ "\n"); raise Match)


(* constants and identifiers *)
fun num x  = (any -- (fn (tok_int n)  => succeed (atom_int n) | _ => fail)) x
fun id x   = (any -- (fn (tok_name i)   => succeed i | _ => fail)) x
fun bool x = (any -- (fn (tok_bool b)   => succeed (atom_bool b) | _ => fail)) x

(* binary operators *)
fun binop x =
      (literal tok_plus return p_plus
    || literal tok_minus return p_minus
    || literal tok_star return p_times
    || literal tok_slash return p_div
    || literal tok_percent return p_mod
    || literal tok_leftleft return p_bit_shift_left
    || literal tok_rightright return p_bit_shift_right
    || literal tok_ampersand return p_bit_and
    || literal tok_bar return p_bit_or
    || literal tok_semicolon return p_effect
    || literal tok_less return p_less
    || literal tok_greater return p_greater
    || literal tok_less_equal return p_less_equal
    || literal tok_greater_equal return p_greater_equal
    || literal tok_comma return p_pair
    || literal tok_equal return p_equals) x
   

(* Precedences and associativities for the binary operators *)
fun prec p_times  = (40,LeftAssoc)   | prec p_div = (40,LeftAssoc)
  | prec p_mod    = (40,LeftAssoc)
  | prec p_plus   = (30,LeftAssoc)   | prec p_minus  = (30,LeftAssoc)
  | prec p_bit_shift_left = (20, LeftAssoc)
  | prec p_bit_shift_right = (20, LeftAssoc)
  | prec p_bit_or = (10, LeftAssoc)
  | prec p_bit_and = (5, LeftAssoc)
  | prec p_equals   = (1, LeftAssoc)
  | prec p_less   = (1, LeftAssoc)
  | prec p_greater   = (1, LeftAssoc)
  | prec p_less_equal   = (1, LeftAssoc)
  | prec p_greater_equal   = (1, LeftAssoc)
  | prec p_effect =   (0, LeftAssoc)
  | prec p_pair =   (0, RightAssoc)


(*
and atyp () =
         literal tok_int_type return t_int
      || literal tok_bool_type return t_bool
      || middle (literal tok_lparen) ($typ) (literal tok_rparen)

and typ () = separate ($atyp) (literal tok_arrow) wth Base.foldr1 t_arrow
*)

(* patterns *)
and apat () = id wth exp_var
      || middle (literal tok_lparen) ($bpat) (literal tok_rparen)
and bpat () = ($apat) -- (fn a => (second (literal tok_comma) ($bpat) wth 
                                    (fn y=>exp_prim(p_pair,[a,y]))
                               || (succeed a)))
and pat () = (($bpat) wth make_pattern)



(* prims *)
and prim0 n p = literal (tok_name n) return (exp_prim(p,[]))
and prim1 n p = second (literal (tok_name n)) ($aexp) wth (fn e=>exp_prim(p, [e]))
and prim2 n p = second (literal (tok_name n)) ($aexp)
     -- (fn (exp_prim(p_pair,[e0,e1]))=>succeed(exp_prim(p, [e0,e1]))
          | _ => fail)


(* "Atomic" expressions *)
and aexp () =
         num wth exp_atom
      || bool wth exp_atom
      || (prim0 "get_store" p_get_store)
      || id wth exp_var
      || middle (literal tok_lparen) ($exp) (literal tok_rparen)
      || (literal tok_lparen) && (literal tok_rparen) wth (fn _ => exp_atom(atom_unit))

and bexp () = $aexp
      || (prim1 "left" p_left)
      || (prim1 "right" p_right)
      || (prim1 "lift" p_lift)
      || (prim1 "debug" p_debug)
      || (prim1 "load_word" p_load_word)
      || (prim2 "store_word" p_store_word)
      || (prim1 "bit_not" p_bit_not)
      || (prim1 "not" p_not)
      || (prim2 "aliases" p_aliases)
      || (prim1 "is_pair" p_is_pair)
      || (prim1 "set_store" p_set_store)
      || (prim1 "effect" p_effect_store)
      || second (literal tok_quote) ($exp) wth exp_quote
      || id && ($aexp) wth (fn(l,a)=>exp_app{memoize=false, label=l, arg=a})
      || second (literal (tok_name "memo")) id && ($aexp) 
         wth (fn(l,a)=>exp_app{memoize=true, label=l, arg=a})

and cexp () = parseinfix (fn (x,b,y) => exp_prim (b,[x,y])) prec ($bexp) binop
           || (($bexp) -- (fn b =>
                second (literal tok_and) ($cexp) wth (fn y=>exp_and(b,y))
             || second (literal tok_or) ($cexp) wth (fn y=>exp_or(b,y))))

and dexp () =
         ($cexp)
      || second (literal tok_if) ($exp) && ($exp) && ($exp) wth (exp_if o flat3)

      || second (literal tok_let) ($pat) && second (literal tok_equal) ($exp)
              && ($exp) wth ((fn (p,e,b)=>exp_let{inline=true, pattern=p, exp=e, body=b}) o flat3)
      || second (literal (tok_name "dlet")) ($pat) && second (literal tok_equal) ($exp)
              && ($exp) wth ((fn (p,e,b)=>exp_let{inline=false, pattern=p, exp=e, body=b}) o flat3)
      || second (literal tok_loop) ($pat) && ($exp)&& ($exp)&& ($exp)&& ($exp)
                     wth (fn (p,(i,(m,(n,e)))) =>
                             exp_loop{index=p, init=i, done=m, next=n, exit=e})

(* The parser for expressions *)
and exp () = $dexp

and def () = ((second (literal tok_fun) id) && ($pat) && (second (literal tok_equal) ($exp))
                    wth (fn (f,(x,e)) => (def_fn(f,x,e,ref []))))
            ||
             ((second (literal tok_val) id) && (second (literal tok_equal) ($exp))
                    wth  (fn (v,e) => (def_val(v, e))))

(* An error-reporting function *)
fun parseerror pos =
    print ("Parsing error at " ^ Position.makestring pos ^ "\n")

fun program x = ((first (repeat1 ($def)) (done())) guard parseerror) x

(* Finally, the parser for Spartan *)
exception ParseError
fun some (SOME x) = x
  | some NONE     = raise ParseError

val parser = some o parse program

end
