(*
  simple - a formal system demonstrating cyclic values
  Copyright (C) 1996  Scott Draves <spot@cs.cmu.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundationeither version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this programif not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)


signature SYNTAX =
sig
    type Name = string
    type myint = Word32.word

    datatype Prim = p_plus | p_minus | p_times | p_equals
              | p_div | p_mod | p_less | p_greater | p_less_equal | p_greater_equal
              | p_bit_and | p_bit_or | p_bit_xor | p_bit_not
              | p_bit_shift_left | p_bit_shift_right
              | p_load_word | p_store_word | p_effect | p_aliases
              | p_is_pair | p_lift | p_debug | p_pair | p_left | p_right
              | p_get_store | p_set_store | p_effect_store | p_not
              | p_is_dynamic

    datatype Atom = atom_int of myint
                  | atom_bool of bool
                  | atom_unit

    datatype Exp = exp_atom of Atom
             | exp_var of Name
             | exp_app of {label:Name, arg:Exp, memoize:bool}
             | exp_loop of {index:M,  init:Exp, done:Exp, next:Exp, exit:Exp}

             | exp_let of {inline:bool, pattern:M, exp:Exp, body:Exp}
             | exp_if  of Exp * Exp * Exp
             | exp_and  of Exp * Exp
             | exp_or  of Exp * Exp
             | exp_prim of Prim * Exp list
             | exp_quote of Exp

             (* for output as C code only *)
             | exp_comma of Exp * Exp
             | exp_assign of Name * Exp
             | exp_while of Exp * Exp


             (* | exp_if2  of Exp * Exp * Exp *)
             (* | exp_abs of Name * Exp *)

     and M = m_static of Atom
           | m_dynamic of Exp
           | m_cyclic of myint * Exp * myint
           | m_pair of M * M
           | m_effect of Exp * M  (* deprecated *)

    type Version = {formal:M, store:M, label:Name, residual:(M*M)}
    type Versions = Version list ref

    datatype Def = def_fn of Name * M * Exp * Versions
                 | def_val of Name * Exp
                 | def_pat of Name * M

    type Program = Def list

    val reify : M -> Exp
    val reify_dynamic_part : M -> Exp option

    val unparse : Exp -> string
    val unparse_m : M -> string
    val unparse_def : Def -> string
    val unparse_prog : Program -> string
    val to_gcc : Program -> string

    val add_type : string -> M -> M
end
