(*
  simple - a formal system demonstrating cyclic values
  Copyright (C) 1996  Scott Draves <spot@cs.cmu.edu>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundationeither version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*)


structure Lex = Lex(structure Token = Token)

structure Parse = Parse(structure Token=Token and Syntax=Syntax)

structure Eval = Eval(structure Syntax=Syntax)

(*
Compiler.Control.Print.printDepth := 1;
*)

fun kbd_prog () = Parse.parser(Lex.lexer(readkeybd ()))

fun string2stream s =
    Position.markstream(Stream.delay (fn () => Base.foldr Stream.cons Stream.empty (map str (explode s))));

fun string2prog s = Parse.parser(Lex.lexer(string2stream s))


fun read_file f = ( print ("parsing "^f^"...\n") ; Parse.parser(Lex.lexer(readfile f)))

fun load_eval () = map use ["eval.sml", "test.sml"]


fun run2file file p store (f, a) =
  (print("----------\n"^f^" "^a^" "^store^" =\n");
   let val s = TextIO.openOut file
       val (v,c) = Eval.run p store f a
   in (print ((Syntax.unparse_m v)^"\n"^(Syntax.unparse_m c)^"\n"));
      TextIO.output(s, (Syntax.unparse_prog p));
      TextIO.closeOut s
   end)

fun debug tv = (Eval.trace_eval := tv;
                Eval.trace_apply := tv;
                Eval.trace_let := tv)

fun test1 () = let val test_p = read_file "test.sal"
                   val run = run2file "out" test_p "store"
               in
		  map run [("asc", "q5"),
			   ("pow", "q5"),
			   ("loopc", "c4"),
                           ("loopc", "c4_2"),
			   ("sum_reduce_signal", "aligned_bytes")
			  ]
	       end ;

fun label_of ((Syntax.m_dynamic (Syntax.exp_app {label,...})),_) = 
    label

fun proto_of prog x y z =
    ("extern word_t "^z^"("^
     Syntax.um(Syntax.add_type "word_t " (Eval.make_formal
		      (Syntax.m_pair((Eval.get_val z prog),
				     (Eval.get_val x prog)))))
     ^");\n")

fun test3() = let
		  val cache_p = (read_file "cache.sal")@(read_file "test2.sal")
                  fun run x y z = 
		      (print ("---"^y^" "^z^"\n");
		       let val rr = Eval.run cache_p x y z
		       in (label_of rr)
		       end)
                  val r0 = run "store1" "sum_reduce_signal" "aligned_bytes"
                  val r00 = run "store2" "sum_reduce_signal" "aligned_12s"
                  val r1 = run "store1_w" "iota_signal" "aligned_bytes"
                  val r2 = run "store2_rw" "copy_signal" "aligned2"
                  val r3 = run "store2_rw_g" "copy_signal" "aligned3"
                  val r33 = run "store3_rrw" "copy_signal" "base64_encode"
                  val r34 = run "store3_ftt" "copy_signal" "base64_decode"
                  val gen = TextIO.openOut "gen.c"
                  val name = TextIO.openOut "names.h"
                  fun def x y = TextIO.output(name, ("#define " ^ x ^ " " ^ y ^ "\n"))
              in
                  TextIO.output (gen, "#include \"runtime.h\"\n");
                  TextIO.output (gen, (Syntax.to_gcc cache_p));
                  TextIO.closeOut gen;
                  
                  def "s_r_s" r0;
                  def "i_s" r1;
                  def "c_s" r2;
                  def "c_s_g" r3;
                  def "s_r_12" r00;
                  def "c_s_6_8" r33;
                  def "c_s_8_6" r34;
                  TextIO.closeOut name
              end

(* this doesn't work because it requires dynamic funciton call *)
fun test_bitcopy() =
              let val prog = (read_file "cache.sal")@(read_file "bitcopy.sal")
                  fun run x y z = (print ("---"^y^"\n"); label_of(Eval.run prog x y z))
                  val r0 = run "store2_rw" "bitcopy" "aligned"
                  val r1 = run "store2_rw" "bitcopy" "aligned4"
                  val gen = TextIO.openOut "gen.c"
                  val name = TextIO.openOut "names.h"
                  fun def x y = TextIO.output(name, ("#define " ^ x ^ " " ^ y ^ "\n"))
              in
                  TextIO.output (gen, "#include \"runtime.h\"\n");
                  TextIO.output (gen, (Syntax.to_gcc prog));
                  TextIO.closeOut gen;
                  
                  def "bitcopy_even" r0;
                  def "bitcopy_even" r1;

                  TextIO.closeOut name
              end

fun do_test srcs tab =
    let val prog = List.concat (map read_file srcs)
	val name = TextIO.openOut "names.h"
	fun run (x, y, z) =
	    let val _ = (print ("---"^x^" "^y^" "^z^"\n"))
		val lab = label_of(Eval.run prog x y z)
	    in
		TextIO.output(name, ("#define " ^ z ^ " " ^ lab ^ "\n"));
		TextIO.output(name, (proto_of prog x y z))
	    end
	val gen = TextIO.openOut "gen.c"
    in
	map run tab;
	TextIO.output (gen, "#include \"runtime.h\"\n");
	TextIO.output (gen, (Syntax.to_gcc prog));
	TextIO.closeOut gen;
	TextIO.closeOut name
    end


fun test8 () = do_test
    ["cache.sal",  "vector.sal",  "image.sal"]
    [("store2_rw", "copy", "rgb2m_1"),
     ("store3_ftt", "copy", "rgb2m_2"),
     ("store4_rrrw", "copy", "rgb2m_3")]

fun test9 () = do_test
    ["cache.sal", "vector.sal"]
    [("store1", "reduce", "sum_shorts"),
     ("store2_wr", "scan", "scan_ints"),
     ("store3_ftt", "binop", "add_shorts"),
     ("store2_rw", "binop", "add_10_shorts"),
     ("store3_ftt", "binop", "add_mapped"),
     ("store3_ftt", "copy", "copy_bin"),
     ("store2_wr", "copy", "copy_filter2"),
     ("store2_wr", "copy", "filter5"),
     ("store2_rw", "copy", "lut1"),
     ("store2_wr", "copy", "wavtab1"),
     ("store1_w", "copy", "fm1"),
     ("store1_w", "copy", "fm2"),
     ("store1_w", "copy", "fm3")]
